from typing import Protocol, runtime_checkable

from preemo.gen.endpoints.batch_create_artifact_part_pb2 import (
    BatchCreateArtifactPartRequest,
    CreateArtifactPartConfig,
    CreateArtifactPartConfigMetadata,
)
from preemo.gen.endpoints.batch_create_artifact_pb2 import (
    BatchCreateArtifactRequest,
    CreateArtifactConfig,
)
from preemo.gen.endpoints.batch_finalize_artifact_pb2 import (
    BatchFinalizeArtifactRequest,
    FinalizeArtifactConfig,
)
from preemo.gen.endpoints.batch_get_artifact_part_pb2 import (
    BatchGetArtifactPartRequest,
    GetArtifactPartConfig,
    GetArtifactPartConfigMetadata,
)
from preemo.gen.endpoints.batch_get_artifact_pb2 import (
    BatchGetArtifactRequest,
    GetArtifactConfig,
)
from preemo.worker._messaging_client import IMessagingClient
from preemo.worker._types import StringValue
from preemo.worker._validation import ensure_keys_match


class ArtifactId(StringValue):
    pass


@runtime_checkable
class IArtifactManager(Protocol):
    def create_artifact(self, content: str) -> ArtifactId:
        pass

    def create_artifacts(self, contents: list[str]) -> list[ArtifactId]:
        pass

    def get_artifact(self, artifact_id: ArtifactId) -> str:
        pass

    def get_artifacts(self, artifact_ids: list[ArtifactId]) -> list[str]:
        pass


class ArtifactManager:
    def __init__(self, *, messaging_client: IMessagingClient) -> None:
        self._messaging_client = messaging_client

    def _create_artifacts(self, *, count: int) -> list[ArtifactId]:
        configs_by_index = {i: CreateArtifactConfig() for i in range(count)}
        response = self._messaging_client.batch_create_artifact(
            BatchCreateArtifactRequest(configs_by_index=configs_by_index)
        )

        return list(
            map(
                lambda x: ArtifactId(value=x[1].artifact_id),
                sorted(response.results_by_index.items(), key=lambda x: x[0]),
            )
        )

    def create_artifact(self, content: str) -> ArtifactId:
        artifact_ids = self.create_artifacts([content])
        if len(artifact_ids) != 1:
            raise Exception("expected exactly one artifact to be created")

        return artifact_ids[0]

    def create_artifacts(self, contents: list[str]) -> list[ArtifactId]:
        artifact_ids = self._create_artifacts(count=len(contents))

        # TODO(adrian@preemo.io, 03/20/2023): handle multipart file upload
        # figure out size for creating multiple parts (100MB?)
        # and decide best way to get size of content len() for now, maybe sys.getsizeof at some point
        configs_by_artifact_id = {
            artifact_id.value: CreateArtifactPartConfig(
                metadatas_by_part_number={1: CreateArtifactPartConfigMetadata()}
            )
            for artifact_id in artifact_ids
        }

        response = self._messaging_client.batch_create_artifact_part(
            BatchCreateArtifactPartRequest(
                configs_by_artifact_id=configs_by_artifact_id
            )
        )

        # TODO(adrian@preemo.io, 03/20/2023): should upload in parallel
        for i, content in enumerate(contents):
            artifact_id = artifact_ids[i].value
            config = configs_by_artifact_id[artifact_id]
            result = response.results_by_artifact_id[artifact_id]

            ensure_keys_match(
                expected=config.metadatas_by_part_number,
                actual=result.metadatas_by_part_number,
            )

            # TODO(adrian@preemo.io, 03/20/2023): this hard-coded part number will need to change for multi-part upload
            metadata = result.metadatas_by_part_number[1]
            if not metadata.HasField("upload_signed_url"):
                raise Exception("expected result metadata to have upload_signed_url")

            # TODO(adrian@preemo.io, 03/20/2023): actually upload to artifact with signed url
            # something like upload_content(content, signed_url)

        self._messaging_client.batch_finalize_artifact(
            BatchFinalizeArtifactRequest(
                configs_by_artifact_id={
                    artifact_id.value: FinalizeArtifactConfig()
                    for artifact_id in artifact_ids
                }
            )
        )

        return artifact_ids

    def get_artifact(self, artifact_id: ArtifactId) -> str:
        contents = self.get_artifacts([artifact_id])
        if len(contents) != 1:
            raise Exception("expected exactly one artifact to be retrieved")

        return contents[0]

    def get_artifacts(self, artifact_ids: list[ArtifactId]) -> list[str]:
        get_artifact_response = self._messaging_client.batch_get_artifact(
            BatchGetArtifactRequest(
                configs_by_artifact_id={
                    artifact_id.value: GetArtifactConfig()
                    for artifact_id in artifact_ids
                }
            )
        )

        configs_by_artifact_id = {
            artifact_id: GetArtifactPartConfig(
                metadatas_by_part_number={
                    (1 + i): GetArtifactPartConfigMetadata()
                    # TODO(adrian@preemo.io, 03/20/2023): don't necessarily want to attempt to
                    # download the entire artifact all at once
                    for i in range(result.part_count)
                }
            )
            for artifact_id, result in get_artifact_response.results_by_artifact_id.items()
        }
        get_artifact_part_response = self._messaging_client.batch_get_artifact_part(
            BatchGetArtifactPartRequest(configs_by_artifact_id=configs_by_artifact_id)
        )

        results: list[str] = []
        for (
            artifact_id,
            result,
        ) in get_artifact_part_response.results_by_artifact_id.items():
            config = configs_by_artifact_id[artifact_id]
            ensure_keys_match(
                expected=config.metadatas_by_part_number,
                actual=result.metadatas_by_part_number,
            )

            # TODO(adrian@preemo.io, 03/20/2023): this hard-coded part number will need to change for multi-part upload
            metadata = result.metadatas_by_part_number[1]
            if not metadata.HasField("download_signed_url"):
                raise Exception("expected result metadata to have download_signed_url")

            # TODO(adrian@preemo.io, 03/20/2023): actually download artifact with signed url
            # something like content = download_content(signed_url)
            # results.append(content)

        return results



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class FinalizeArtifactConfig(google.protobuf.message.Message):
    """This config is included for potential future use."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___FinalizeArtifactConfig = FinalizeArtifactConfig

@typing_extensions.final
class BatchFinalizeArtifactRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ConfigsByArtifactIdEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___FinalizeArtifactConfig: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___FinalizeArtifactConfig | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    CONFIGS_BY_ARTIFACT_ID_FIELD_NUMBER: builtins.int
    @property
    def configs_by_artifact_id(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___FinalizeArtifactConfig]: ...
    def __init__(
        self,
        *,
        configs_by_artifact_id: collections.abc.Mapping[builtins.str, global___FinalizeArtifactConfig] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["configs_by_artifact_id", b"configs_by_artifact_id"]) -> None: ...

global___BatchFinalizeArtifactRequest = BatchFinalizeArtifactRequest

@typing_extensions.final
class FinalizeArtifactResult(google.protobuf.message.Message):
    """This result is included for potential future use."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___FinalizeArtifactResult = FinalizeArtifactResult

@typing_extensions.final
class BatchFinalizeArtifactResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ResultsByArtifactIdEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___FinalizeArtifactResult: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___FinalizeArtifactResult | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    RESULTS_BY_ARTIFACT_ID_FIELD_NUMBER: builtins.int
    @property
    def results_by_artifact_id(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___FinalizeArtifactResult]: ...
    def __init__(
        self,
        *,
        results_by_artifact_id: collections.abc.Mapping[builtins.str, global___FinalizeArtifactResult] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["results_by_artifact_id", b"results_by_artifact_id"]) -> None: ...

global___BatchFinalizeArtifactResponse = BatchFinalizeArtifactResponse



from typing import Callable, Optional

from google.protobuf.struct_pb2 import NULL_VALUE

from preemo.gen.endpoints.check_function_pb2 import CheckFunctionRequest
from preemo.gen.endpoints.execute_function_pb2 import ExecuteFunctionRequest
from preemo.gen.endpoints.register_function_pb2 import RegisterFunctionRequest
from preemo.gen.models.registered_function_pb2 import RegisteredFunction
from preemo.gen.models.value_pb2 import Value
from preemo.worker._artifact_manager import ArtifactId, IArtifactManager
from preemo.worker._function_registry import FunctionRegistry
from preemo.worker._messaging_client import IMessagingClient


class Function:
    def __init__(
        self,
        *,
        artifact_manager: IArtifactManager,
        messaging_client: IMessagingClient,
        name: str,
        namespace: Optional[str],
    ) -> None:
        self._artifact_manager = artifact_manager
        self._messaging_client = messaging_client
        self.name = name
        self.namespace = namespace

        self._ensure_function_is_registered()

    def _ensure_function_is_registered(self) -> None:
        # check_function raises an error if the function is not found
        self._messaging_client.check_function(
            CheckFunctionRequest(
                function_to_check=RegisteredFunction(
                    name=self.name, namespace=self.namespace
                )
            )
        )

    def __call__(self, params: Optional[str] = None) -> Optional[str]:
        if params is None:
            function_parameter = Value(null_value=NULL_VALUE)
        else:
            artifact_id = self._artifact_manager.create_artifact(params)
            function_parameter = Value(artifact_id=artifact_id.value)

        response = self._messaging_client.execute_function(
            ExecuteFunctionRequest(
                function_to_execute=RegisteredFunction(
                    name=self.name, namespace=self.namespace
                ),
                parameters_by_index={0: function_parameter},
            )
        )

        function_result = response.results_by_index[0]

        match function_result.WhichOneof("kind"):
            case "null_value":
                return None
            case "artifact_id":
                return self._artifact_manager.get_artifact(
                    ArtifactId(value=function_result.artifact_id)
                )
            case _:
                raise Exception(
                    f"received unexpected function_result: {function_result}"
                )


class WorkerClient:
    def __init__(
        self, *, artifact_manager: IArtifactManager, messaging_client: IMessagingClient
    ) -> None:
        self._artifact_manager = artifact_manager
        self._messaging_client = messaging_client

        self._function_registry = FunctionRegistry()

    def get_function(self, name: str, *, namespace: Optional[str] = None) -> Function:
        return Function(
            artifact_manager=self._artifact_manager,
            messaging_client=self._messaging_client,
            name=name,
            namespace=namespace,
        )

    def parallelize(
        self,
        function: Function,
        *,
        params: Optional[list[str]] = None,
        count: Optional[int] = None,
    ) -> list[Optional[str]]:
        # TODO(adrian@preemo.io, 03/20/2023): should take an optional config argument includes stuff like max batch size

        if params is None:
            if count is None:
                raise ValueError("either params or count must be specified")

            if count <= 0:
                raise ValueError("count must be positive")

            function_parameters_by_index = {
                i: Value(null_value=NULL_VALUE) for i in range(count)
            }
        else:
            if count is not None:
                raise ValueError("params and count must not both be specified")

            if len(params) == 0:
                return []

            artifact_ids = self._artifact_manager.create_artifacts(params)
            function_parameters_by_index = {
                i: Value(artifact_id=artifact_id.value)
                for i, artifact_id in enumerate(artifact_ids)
            }

        response = self._messaging_client.execute_function(
            ExecuteFunctionRequest(
                function_to_execute=RegisteredFunction(
                    name=function.name, namespace=function.namespace
                ),
                parameters_by_index=function_parameters_by_index,
            )
        )

        # TODO(adrian@preemo.io, 03/20/2023): should download results in parallel or return a handle that allows the user to download result
        results: list[Optional[str]] = []
        for _, function_result in sorted(
            response.results_by_index.items(), key=lambda x: x[0]
        ):
            match function_result.WhichOneof("kind"):
                case "null_value":
                    results.append(None)
                case "artifact_id":
                    results.append(
                        self._artifact_manager.get_artifact(
                            ArtifactId(value=function_result.artifact_id)
                        )
                    )
                case _:
                    raise Exception(
                        f"received unexpected function_result: {function_result}"
                    )

        return results

    def register(
        self,
        outer_function: Optional[Callable] = None,
        *,
        name: Optional[str] = None,
        namespace: Optional[str] = None,
    ) -> Callable:
        def decorator(function: Callable) -> Callable:
            if name is None:
                function_name = function.__name__
            else:
                function_name = name

            self._function_registry.register_function(
                function, name=function_name, namespace=namespace
            )

            self._messaging_client.register_function(
                RegisterFunctionRequest(
                    function_to_register=RegisteredFunction(
                        name=function_name, namespace=namespace
                    )
                )
            )

            return function

        if outer_function is None:
            return decorator

        return decorator(outer_function)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: endpoints/batch_create_artifact_part.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*endpoints/batch_create_artifact_part.proto\x12\tendpoints\"\"\n CreateArtifactPartConfigMetadata\"\xe7\x01\n\x18\x43reateArtifactPartConfig\x12`\n\x18metadatas_by_part_number\x18\x01 \x03(\x0b\x32>.endpoints.CreateArtifactPartConfig.MetadatasByPartNumberEntry\x1ai\n\x1aMetadatasByPartNumberEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.endpoints.CreateArtifactPartConfigMetadata:\x02\x38\x01\"\xe5\x01\n\x1e\x42\x61tchCreateArtifactPartRequest\x12\x62\n\x16\x63onfigs_by_artifact_id\x18\x01 \x03(\x0b\x32\x42.endpoints.BatchCreateArtifactPartRequest.ConfigsByArtifactIdEntry\x1a_\n\x18\x43onfigsByArtifactIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x32\n\x05value\x18\x02 \x01(\x0b\x32#.endpoints.CreateArtifactPartConfig:\x02\x38\x01\"X\n CreateArtifactPartResultMetadata\x12\x1e\n\x11upload_signed_url\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x14\n\x12_upload_signed_url\"\xe7\x01\n\x18\x43reateArtifactPartResult\x12`\n\x18metadatas_by_part_number\x18\x01 \x03(\x0b\x32>.endpoints.CreateArtifactPartResult.MetadatasByPartNumberEntry\x1ai\n\x1aMetadatasByPartNumberEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.endpoints.CreateArtifactPartResultMetadata:\x02\x38\x01\"\xe7\x01\n\x1f\x42\x61tchCreateArtifactPartResponse\x12\x63\n\x16results_by_artifact_id\x18\x01 \x03(\x0b\x32\x43.endpoints.BatchCreateArtifactPartResponse.ResultsByArtifactIdEntry\x1a_\n\x18ResultsByArtifactIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x32\n\x05value\x18\x02 \x01(\x0b\x32#.endpoints.CreateArtifactPartResult:\x02\x38\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'endpoints.batch_create_artifact_part_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CREATEARTIFACTPARTCONFIG_METADATASBYPARTNUMBERENTRY._options = None
  _CREATEARTIFACTPARTCONFIG_METADATASBYPARTNUMBERENTRY._serialized_options = b'8\001'
  _BATCHCREATEARTIFACTPARTREQUEST_CONFIGSBYARTIFACTIDENTRY._options = None
  _BATCHCREATEARTIFACTPARTREQUEST_CONFIGSBYARTIFACTIDENTRY._serialized_options = b'8\001'
  _CREATEARTIFACTPARTRESULT_METADATASBYPARTNUMBERENTRY._options = None
  _CREATEARTIFACTPARTRESULT_METADATASBYPARTNUMBERENTRY._serialized_options = b'8\001'
  _BATCHCREATEARTIFACTPARTRESPONSE_RESULTSBYARTIFACTIDENTRY._options = None
  _BATCHCREATEARTIFACTPARTRESPONSE_RESULTSBYARTIFACTIDENTRY._serialized_options = b'8\001'
  _CREATEARTIFACTPARTCONFIGMETADATA._serialized_start=57
  _CREATEARTIFACTPARTCONFIGMETADATA._serialized_end=91
  _CREATEARTIFACTPARTCONFIG._serialized_start=94
  _CREATEARTIFACTPARTCONFIG._serialized_end=325
  _CREATEARTIFACTPARTCONFIG_METADATASBYPARTNUMBERENTRY._serialized_start=220
  _CREATEARTIFACTPARTCONFIG_METADATASBYPARTNUMBERENTRY._serialized_end=325
  _BATCHCREATEARTIFACTPARTREQUEST._serialized_start=328
  _BATCHCREATEARTIFACTPARTREQUEST._serialized_end=557
  _BATCHCREATEARTIFACTPARTREQUEST_CONFIGSBYARTIFACTIDENTRY._serialized_start=462
  _BATCHCREATEARTIFACTPARTREQUEST_CONFIGSBYARTIFACTIDENTRY._serialized_end=557
  _CREATEARTIFACTPARTRESULTMETADATA._serialized_start=559
  _CREATEARTIFACTPARTRESULTMETADATA._serialized_end=647
  _CREATEARTIFACTPARTRESULT._serialized_start=650
  _CREATEARTIFACTPARTRESULT._serialized_end=881
  _CREATEARTIFACTPARTRESULT_METADATASBYPARTNUMBERENTRY._serialized_start=776
  _CREATEARTIFACTPARTRESULT_METADATASBYPARTNUMBERENTRY._serialized_end=881
  _BATCHCREATEARTIFACTPARTRESPONSE._serialized_start=884
  _BATCHCREATEARTIFACTPARTRESPONSE._serialized_end=1115
  _BATCHCREATEARTIFACTPARTRESPONSE_RESULTSBYARTIFACTIDENTRY._serialized_start=1020
  _BATCHCREATEARTIFACTPARTRESPONSE_RESULTSBYARTIFACTIDENTRY._serialized_end=1115
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: endpoints/header.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x65ndpoints/header.proto\x12\tendpoints\"1\n\rHeaderRequest\x12\x14\n\x07version\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\n\n\x08_version\"\x10\n\x0eHeaderResponseb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'endpoints.header_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HEADERREQUEST._serialized_start=37
  _HEADERREQUEST._serialized_end=86
  _HEADERRESPONSE._serialized_start=88
  _HEADERRESPONSE._serialized_end=104
# @@protoc_insertion_point(module_scope)



from typing import NoReturn

from pydantic import BaseModel, StrictStr

_ASSERT_NEVER_REPR_MAX_LENGTH = 100


# Copied from https://github.com/python/cpython/blob/main/Lib/typing.py#L2467
# typing.assert_never and typing.Never were added in python 3.11, but NoReturn is functionally equivalent in prior versions
def assert_never(arg: NoReturn, /) -> NoReturn:
    """Statically assert that a line of code is unreachable.
    Example::
        def int_or_str(arg: int | str) -> None:
            match arg:
                case int():
                    print("It's an int")
                case str():
                    print("It's a str")
                case _:
                    assert_never(arg)
    If a type checker finds that a call to assert_never() is
    reachable, it will emit an error.
    At runtime, this throws an exception when called.
    """
    value = repr(arg)
    if len(value) > _ASSERT_NEVER_REPR_MAX_LENGTH:
        value = value[:_ASSERT_NEVER_REPR_MAX_LENGTH] + "..."
    raise AssertionError(f"Expected code to be unreachable, but got: {value}")


class ImmutableModel(BaseModel):
    class Config:
        allow_mutation = False


class StringValue(ImmutableModel):
    value: StrictStr



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import google.protobuf.struct_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Value(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NULL_VALUE_FIELD_NUMBER: builtins.int
    ARTIFACT_ID_FIELD_NUMBER: builtins.int
    null_value: google.protobuf.struct_pb2.NullValue.ValueType
    artifact_id: builtins.str
    """Unique identifier for a file stored in the cloud."""
    def __init__(
        self,
        *,
        null_value: google.protobuf.struct_pb2.NullValue.ValueType = ...,
        artifact_id: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["artifact_id", b"artifact_id", "kind", b"kind", "null_value", b"null_value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["artifact_id", b"artifact_id", "kind", b"kind", "null_value", b"null_value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["kind", b"kind"]) -> typing_extensions.Literal["null_value", "artifact_id"] | None: ...

global___Value = Value



from preemo.worker._artifact_manager import ArtifactManager as _ArtifactManager
from preemo.worker._env import get_optional_env as _get_optional_env
from preemo.worker._messaging_client import IMessagingClient as _IMessagingClient
from preemo.worker._messaging_client import (
    LocalMessagingClient as _LocalMessagingClient,
)
from preemo.worker._messaging_client import MessagingClient as _MessagingClient
from preemo.worker._worker_client import WorkerClient as _WorkerClient

__all__ = ["get_function", "parallelize", "register"]


def _construct_messaging_client() -> _IMessagingClient:
    worker_server_url = _get_optional_env("PREEMO_WORKER_SERVER_URL")

    if worker_server_url is None:
        return _LocalMessagingClient()

    return _MessagingClient(worker_server_url=worker_server_url)


_messaging_client = _construct_messaging_client()
_artifact_manager = _ArtifactManager(messaging_client=_messaging_client)
_worker_client = _WorkerClient(
    artifact_manager=_artifact_manager, messaging_client=_messaging_client
)

get_function = _worker_client.get_function
parallelize = _worker_client.parallelize
register = _worker_client.register



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import preemo.gen.models.registered_function_pb2
import preemo.gen.models.value_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ExecuteFunctionRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ParametersByIndexEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> preemo.gen.models.value_pb2.Value: ...
        def __init__(
            self,
            *,
            key: builtins.int = ...,
            value: preemo.gen.models.value_pb2.Value | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    FUNCTION_TO_EXECUTE_FIELD_NUMBER: builtins.int
    PARAMETERS_BY_INDEX_FIELD_NUMBER: builtins.int
    @property
    def function_to_execute(self) -> preemo.gen.models.registered_function_pb2.RegisteredFunction: ...
    @property
    def parameters_by_index(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, preemo.gen.models.value_pb2.Value]: ...
    def __init__(
        self,
        *,
        function_to_execute: preemo.gen.models.registered_function_pb2.RegisteredFunction | None = ...,
        parameters_by_index: collections.abc.Mapping[builtins.int, preemo.gen.models.value_pb2.Value] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_function_to_execute", b"_function_to_execute", "function_to_execute", b"function_to_execute"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_function_to_execute", b"_function_to_execute", "function_to_execute", b"function_to_execute", "parameters_by_index", b"parameters_by_index"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_function_to_execute", b"_function_to_execute"]) -> typing_extensions.Literal["function_to_execute"] | None: ...

global___ExecuteFunctionRequest = ExecuteFunctionRequest

@typing_extensions.final
class ExecuteFunctionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ResultsByIndexEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> preemo.gen.models.value_pb2.Value: ...
        def __init__(
            self,
            *,
            key: builtins.int = ...,
            value: preemo.gen.models.value_pb2.Value | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    RESULTS_BY_INDEX_FIELD_NUMBER: builtins.int
    @property
    def results_by_index(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, preemo.gen.models.value_pb2.Value]: ...
    def __init__(
        self,
        *,
        results_by_index: collections.abc.Mapping[builtins.int, preemo.gen.models.value_pb2.Value] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["results_by_index", b"results_by_index"]) -> None: ...

global___ExecuteFunctionResponse = ExecuteFunctionResponse



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: endpoints/check_function.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from models import registered_function_pb2 as models_dot_registered__function__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x65ndpoints/check_function.proto\x12\tendpoints\x1a models/registered_function.proto\"h\n\x14\x43heckFunctionRequest\x12:\n\x11\x66unction_to_check\x18\x01 \x01(\x0b\x32\x1a.models.RegisteredFunctionH\x00\x88\x01\x01\x42\x14\n\x12_function_to_check\"\x17\n\x15\x43heckFunctionResponseb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'endpoints.check_function_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CHECKFUNCTIONREQUEST._serialized_start=79
  _CHECKFUNCTIONREQUEST._serialized_end=183
  _CHECKFUNCTIONRESPONSE._serialized_start=185
  _CHECKFUNCTIONRESPONSE._serialized_end=208
# @@protoc_insertion_point(module_scope)



from typing import Callable, Dict, Optional


class FunctionRegistry:
    def __init__(self) -> None:
        self._global_functions_by_name: Dict[str, Callable] = {}
        self._functions_by_namespace_and_name: Dict[str, Dict[str, Callable]] = {}

    def register_function(
        self, function: Callable, *, name: str, namespace: Optional[str] = None
    ) -> None:
        if namespace is None:
            if name in self._global_functions_by_name:
                raise Exception(
                    f"must not register multiple functions with the same name: {name}"
                )
            self._global_functions_by_name[name] = function

            return

        if namespace not in self._functions_by_namespace_and_name:
            self._functions_by_namespace_and_name[namespace] = {}
        functions_by_name = self._functions_by_namespace_and_name[namespace]

        if name in functions_by_name:
            raise Exception(
                f"must not register multiple functions with the same namespace: {namespace} and name: {name}"
            )
        functions_by_name[name] = function

    def get_function(self, *, name: str, namespace: Optional[str] = None) -> Callable:
        if namespace is None:
            function = self._global_functions_by_name.get(name)
            if function is None:
                raise Exception(f"cannot find registered function with name: {name}")

            return function

        function = self._functions_by_namespace_and_name.get(namespace, {}).get(name)
        if function is None:
            raise Exception(
                f"cannot find registered function with namespace: {namespace} and name: {name}"
            )

        return function



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class HeaderRequest(google.protobuf.message.Message):
    """Initial request that should be made to establish connection.
    Intended to contain any and all metadata the server might need.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERSION_FIELD_NUMBER: builtins.int
    version: builtins.str
    def __init__(
        self,
        *,
        version: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_version", b"_version", "version", b"version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_version", b"_version", "version", b"version"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_version", b"_version"]) -> typing_extensions.Literal["version"] | None: ...

global___HeaderRequest = HeaderRequest

@typing_extensions.final
class HeaderResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___HeaderResponse = HeaderResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class CreateArtifactConfig(google.protobuf.message.Message):
    """This config is included for potential future use.
    In the future it may include part count, expiration, or other metadata.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CreateArtifactConfig = CreateArtifactConfig

@typing_extensions.final
class BatchCreateArtifactRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ConfigsByIndexEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> global___CreateArtifactConfig: ...
        def __init__(
            self,
            *,
            key: builtins.int = ...,
            value: global___CreateArtifactConfig | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    CONFIGS_BY_INDEX_FIELD_NUMBER: builtins.int
    @property
    def configs_by_index(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___CreateArtifactConfig]:
        """This field is a map instead of an array to avoid depending on the
        response array being in the same order as the request array.
        """
    def __init__(
        self,
        *,
        configs_by_index: collections.abc.Mapping[builtins.int, global___CreateArtifactConfig] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["configs_by_index", b"configs_by_index"]) -> None: ...

global___BatchCreateArtifactRequest = BatchCreateArtifactRequest

@typing_extensions.final
class CreateArtifactResult(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ARTIFACT_ID_FIELD_NUMBER: builtins.int
    artifact_id: builtins.str
    def __init__(
        self,
        *,
        artifact_id: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_artifact_id", b"_artifact_id", "artifact_id", b"artifact_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_artifact_id", b"_artifact_id", "artifact_id", b"artifact_id"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_artifact_id", b"_artifact_id"]) -> typing_extensions.Literal["artifact_id"] | None: ...

global___CreateArtifactResult = CreateArtifactResult

@typing_extensions.final
class BatchCreateArtifactResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ResultsByIndexEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> global___CreateArtifactResult: ...
        def __init__(
            self,
            *,
            key: builtins.int = ...,
            value: global___CreateArtifactResult | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    RESULTS_BY_INDEX_FIELD_NUMBER: builtins.int
    @property
    def results_by_index(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___CreateArtifactResult]: ...
    def __init__(
        self,
        *,
        results_by_index: collections.abc.Mapping[builtins.int, global___CreateArtifactResult] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["results_by_index", b"results_by_index"]) -> None: ...

global___BatchCreateArtifactResponse = BatchCreateArtifactResponse



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: endpoints/batch_create_artifact.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%endpoints/batch_create_artifact.proto\x12\tendpoints\"\x16\n\x14\x43reateArtifactConfig\"\xc9\x01\n\x1a\x42\x61tchCreateArtifactRequest\x12S\n\x10\x63onfigs_by_index\x18\x01 \x03(\x0b\x32\x39.endpoints.BatchCreateArtifactRequest.ConfigsByIndexEntry\x1aV\n\x13\x43onfigsByIndexEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12.\n\x05value\x18\x02 \x01(\x0b\x32\x1f.endpoints.CreateArtifactConfig:\x02\x38\x01\"@\n\x14\x43reateArtifactResult\x12\x18\n\x0b\x61rtifact_id\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0e\n\x0c_artifact_id\"\xcb\x01\n\x1b\x42\x61tchCreateArtifactResponse\x12T\n\x10results_by_index\x18\x01 \x03(\x0b\x32:.endpoints.BatchCreateArtifactResponse.ResultsByIndexEntry\x1aV\n\x13ResultsByIndexEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12.\n\x05value\x18\x02 \x01(\x0b\x32\x1f.endpoints.CreateArtifactResult:\x02\x38\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'endpoints.batch_create_artifact_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _BATCHCREATEARTIFACTREQUEST_CONFIGSBYINDEXENTRY._options = None
  _BATCHCREATEARTIFACTREQUEST_CONFIGSBYINDEXENTRY._serialized_options = b'8\001'
  _BATCHCREATEARTIFACTRESPONSE_RESULTSBYINDEXENTRY._options = None
  _BATCHCREATEARTIFACTRESPONSE_RESULTSBYINDEXENTRY._serialized_options = b'8\001'
  _CREATEARTIFACTCONFIG._serialized_start=52
  _CREATEARTIFACTCONFIG._serialized_end=74
  _BATCHCREATEARTIFACTREQUEST._serialized_start=77
  _BATCHCREATEARTIFACTREQUEST._serialized_end=278
  _BATCHCREATEARTIFACTREQUEST_CONFIGSBYINDEXENTRY._serialized_start=192
  _BATCHCREATEARTIFACTREQUEST_CONFIGSBYINDEXENTRY._serialized_end=278
  _CREATEARTIFACTRESULT._serialized_start=280
  _CREATEARTIFACTRESULT._serialized_end=344
  _BATCHCREATEARTIFACTRESPONSE._serialized_start=347
  _BATCHCREATEARTIFACTRESPONSE._serialized_end=550
  _BATCHCREATEARTIFACTRESPONSE_RESULTSBYINDEXENTRY._serialized_start=464
  _BATCHCREATEARTIFACTRESPONSE_RESULTSBYINDEXENTRY._serialized_end=550
# @@protoc_insertion_point(module_scope)



import os
from typing import Optional


def get_optional_env(name: str) -> Optional[str]:
    value = os.getenv(key=name)
    if value is None or len(value) == 0:
        return None

    return value


def get_required_env(name: str) -> str:
    value = get_optional_env(name)
    if value is None:
        raise Exception(f"missing required env variable: {name}")

    return value



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: endpoints/execute_function.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from models import registered_function_pb2 as models_dot_registered__function__pb2
from models import value_pb2 as models_dot_value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n endpoints/execute_function.proto\x12\tendpoints\x1a models/registered_function.proto\x1a\x12models/value.proto\"\x8e\x02\n\x16\x45xecuteFunctionRequest\x12<\n\x13\x66unction_to_execute\x18\x01 \x01(\x0b\x32\x1a.models.RegisteredFunctionH\x00\x88\x01\x01\x12U\n\x13parameters_by_index\x18\x02 \x03(\x0b\x32\x38.endpoints.ExecuteFunctionRequest.ParametersByIndexEntry\x1aG\n\x16ParametersByIndexEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\x1c\n\x05value\x18\x02 \x01(\x0b\x32\r.models.Value:\x02\x38\x01\x42\x16\n\x14_function_to_execute\"\xb1\x01\n\x17\x45xecuteFunctionResponse\x12P\n\x10results_by_index\x18\x01 \x03(\x0b\x32\x36.endpoints.ExecuteFunctionResponse.ResultsByIndexEntry\x1a\x44\n\x13ResultsByIndexEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\x1c\n\x05value\x18\x02 \x01(\x0b\x32\r.models.Value:\x02\x38\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'endpoints.execute_function_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _EXECUTEFUNCTIONREQUEST_PARAMETERSBYINDEXENTRY._options = None
  _EXECUTEFUNCTIONREQUEST_PARAMETERSBYINDEXENTRY._serialized_options = b'8\001'
  _EXECUTEFUNCTIONRESPONSE_RESULTSBYINDEXENTRY._options = None
  _EXECUTEFUNCTIONRESPONSE_RESULTSBYINDEXENTRY._serialized_options = b'8\001'
  _EXECUTEFUNCTIONREQUEST._serialized_start=102
  _EXECUTEFUNCTIONREQUEST._serialized_end=372
  _EXECUTEFUNCTIONREQUEST_PARAMETERSBYINDEXENTRY._serialized_start=277
  _EXECUTEFUNCTIONREQUEST_PARAMETERSBYINDEXENTRY._serialized_end=348
  _EXECUTEFUNCTIONRESPONSE._serialized_start=375
  _EXECUTEFUNCTIONRESPONSE._serialized_end=552
  _EXECUTEFUNCTIONRESPONSE_RESULTSBYINDEXENTRY._serialized_start=484
  _EXECUTEFUNCTIONRESPONSE_RESULTSBYINDEXENTRY._serialized_end=552
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: endpoints/batch_get_artifact_part.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'endpoints/batch_get_artifact_part.proto\x12\tendpoints\"\x1f\n\x1dGetArtifactPartConfigMetadata\"\xde\x01\n\x15GetArtifactPartConfig\x12]\n\x18metadatas_by_part_number\x18\x01 \x03(\x0b\x32;.endpoints.GetArtifactPartConfig.MetadatasByPartNumberEntry\x1a\x66\n\x1aMetadatasByPartNumberEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\x37\n\x05value\x18\x02 \x01(\x0b\x32(.endpoints.GetArtifactPartConfigMetadata:\x02\x38\x01\"\xdc\x01\n\x1b\x42\x61tchGetArtifactPartRequest\x12_\n\x16\x63onfigs_by_artifact_id\x18\x01 \x03(\x0b\x32?.endpoints.BatchGetArtifactPartRequest.ConfigsByArtifactIdEntry\x1a\\\n\x18\x43onfigsByArtifactIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12/\n\x05value\x18\x02 \x01(\x0b\x32 .endpoints.GetArtifactPartConfig:\x02\x38\x01\"Y\n\x1dGetArtifactPartResultMetadata\x12 \n\x13\x64ownload_signed_url\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x16\n\x14_download_signed_url\"\xde\x01\n\x15GetArtifactPartResult\x12]\n\x18metadatas_by_part_number\x18\x01 \x03(\x0b\x32;.endpoints.GetArtifactPartResult.MetadatasByPartNumberEntry\x1a\x66\n\x1aMetadatasByPartNumberEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\x37\n\x05value\x18\x02 \x01(\x0b\x32(.endpoints.GetArtifactPartResultMetadata:\x02\x38\x01\"\xde\x01\n\x1c\x42\x61tchGetArtifactPartResponse\x12`\n\x16results_by_artifact_id\x18\x01 \x03(\x0b\x32@.endpoints.BatchGetArtifactPartResponse.ResultsByArtifactIdEntry\x1a\\\n\x18ResultsByArtifactIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12/\n\x05value\x18\x02 \x01(\x0b\x32 .endpoints.GetArtifactPartResult:\x02\x38\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'endpoints.batch_get_artifact_part_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _GETARTIFACTPARTCONFIG_METADATASBYPARTNUMBERENTRY._options = None
  _GETARTIFACTPARTCONFIG_METADATASBYPARTNUMBERENTRY._serialized_options = b'8\001'
  _BATCHGETARTIFACTPARTREQUEST_CONFIGSBYARTIFACTIDENTRY._options = None
  _BATCHGETARTIFACTPARTREQUEST_CONFIGSBYARTIFACTIDENTRY._serialized_options = b'8\001'
  _GETARTIFACTPARTRESULT_METADATASBYPARTNUMBERENTRY._options = None
  _GETARTIFACTPARTRESULT_METADATASBYPARTNUMBERENTRY._serialized_options = b'8\001'
  _BATCHGETARTIFACTPARTRESPONSE_RESULTSBYARTIFACTIDENTRY._options = None
  _BATCHGETARTIFACTPARTRESPONSE_RESULTSBYARTIFACTIDENTRY._serialized_options = b'8\001'
  _GETARTIFACTPARTCONFIGMETADATA._serialized_start=54
  _GETARTIFACTPARTCONFIGMETADATA._serialized_end=85
  _GETARTIFACTPARTCONFIG._serialized_start=88
  _GETARTIFACTPARTCONFIG._serialized_end=310
  _GETARTIFACTPARTCONFIG_METADATASBYPARTNUMBERENTRY._serialized_start=208
  _GETARTIFACTPARTCONFIG_METADATASBYPARTNUMBERENTRY._serialized_end=310
  _BATCHGETARTIFACTPARTREQUEST._serialized_start=313
  _BATCHGETARTIFACTPARTREQUEST._serialized_end=533
  _BATCHGETARTIFACTPARTREQUEST_CONFIGSBYARTIFACTIDENTRY._serialized_start=441
  _BATCHGETARTIFACTPARTREQUEST_CONFIGSBYARTIFACTIDENTRY._serialized_end=533
  _GETARTIFACTPARTRESULTMETADATA._serialized_start=535
  _GETARTIFACTPARTRESULTMETADATA._serialized_end=624
  _GETARTIFACTPARTRESULT._serialized_start=627
  _GETARTIFACTPARTRESULT._serialized_end=849
  _GETARTIFACTPARTRESULT_METADATASBYPARTNUMBERENTRY._serialized_start=747
  _GETARTIFACTPARTRESULT_METADATASBYPARTNUMBERENTRY._serialized_end=849
  _BATCHGETARTIFACTPARTRESPONSE._serialized_start=852
  _BATCHGETARTIFACTPARTRESPONSE._serialized_end=1074
  _BATCHGETARTIFACTPARTRESPONSE_RESULTSBYARTIFACTIDENTRY._serialized_start=982
  _BATCHGETARTIFACTPARTRESPONSE_RESULTSBYARTIFACTIDENTRY._serialized_end=1074
# @@protoc_insertion_point(module_scope)
